/*
 * psp102.analogfunction.c - analog function implementations
 *
 * This is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 */

// #if HAVE_CONFIG_H
// # include <config.h>
// #endif

#include <stdio.h>
#include "psp102.analogfunction.h"
#include <math.h>


/* math functions and appropriate derivatives */
static inline double _fn_cos(double arg)          { return  cos(arg); }
static inline double _d0_cos(double arg)          { return (-sin(arg)); }
static inline double _fn_sin(double arg)          { return  sin(arg); }
static inline double _d0_sin(double arg)          { return (cos(arg)); }
static inline double _fn_tan(double arg)          { return  tan(arg); }
static inline double _d0_tan(double arg)          { return (1.0/cos(arg)/cos(arg)); }
static inline double _fn_cosh(double arg)         { return  cosh(arg); }
static inline double _d0_cosh(double arg)         { return (sinh(arg)); }
static inline double _fn_sinh(double arg)         { return  sinh(arg); }
static inline double _d0_sinh(double arg)         { return (cosh(arg)); }
static inline double _fn_tanh(double arg)         { return  tanh(arg); }
static inline double _d0_tanh(double arg)         { return (1.0/cosh(arg)/cosh(arg)); }
static inline double _fn_acos(double arg)         { return  acos(arg); }
static inline double _d0_acos(double arg)         { return (-1.0/sqrt(1-arg*arg)); }
static inline double _fn_asin(double arg)         { return  asin(arg); }
static inline double _d0_asin(double arg)         { return (+1.0/sqrt(1-arg*arg)); }
static inline double _fn_atan(double arg)         { return  atan(arg); }
static inline double _d0_atan(double arg)         { return (+1.0/(1+arg*arg)); }
static inline double _fn_acosh(double arg)        { return  acosh(arg); }
static inline double _d0_acosh(double arg)        { return (1.0/(sqrt(arg-1)*sqrt(arg+1))); }
static inline double _fn_asinh(double arg)        { return  asinh(arg); }
static inline double _d0_asinh(double arg)        { return (1.0/(sqrt(arg*arg+1))); }
static inline double _fn_atanh(double arg)        { return  atanh(arg); }
static inline double _d0_atanh(double arg)        { return (+1.0/(1-arg*arg)); }

static inline double _fn_logE(double arg)         { return  log(arg); }
static inline double _d0_logE(double arg)         { return (1.0/arg); }
static inline double _fn_log10(double arg)        { return  log10(arg); }
static inline double _d0_log10(double arg)        { return (1.0/arg/log(10.0)); }
static inline double _fn_exp(double arg)          { return  exp(arg); }
static inline double _d0_exp(double arg)          { return exp(arg); }
static inline double _fn_sqrt(double arg)         { return  sqrt(arg); }
static inline double _d0_sqrt(double arg)         { return (1.0/sqrt(arg)/2.0); }

static inline double _fn_abs(double arg)          { return fabs(arg); }
static inline double _d0_abs(double arg)          { return (((arg)>=0)?(+1.0):(-1.0)); }

static inline double _fn_floor(double arg)        { return  floor(arg); }
static inline double _d0_floor(double arg)        { (void) arg; return 0.0; }

static inline double _fn_ceil(double arg)         { return  ceil(arg); }
static inline double _d0_ceil(double arg)         { (void) arg; return 0.0; }

static inline double _fn_hypot(double x,double y) { return sqrt((x)*(x)+(y)*(y)); }
static inline double _d0_hypot(double x,double y) { return (x)/sqrt((x)*(x)+(y)*(y)); }
static inline double _d1_hypot(double x,double y) { return (y)/sqrt((x)*(x)+(y)*(y)); }

static inline double _fn_atan2(double x,double y) { return atan2(x,y); }
static inline double _d0_atan2(double x,double y) { return -y / (x*x + y*y); }
static inline double _d1_atan2(double x,double y) { return  x / (x*x + y*y); }

static inline double _fn_max(double x,double y)   { return ((x)>(y))?(x):(y); }
static inline double _d0_max(double x,double y)   { return ((x)>(y))?1.0:0.0; }
static inline double _d1_max(double x,double y)   { return ((x)>(y))?0.0:1.0; }

static inline double _fn_min(double x,double y)   { return ((x)<(y))?(x):(y); }
static inline double _d0_min(double x,double y)   { return ((x)<(y))?1.0:0.0; }
static inline double _d1_min(double x,double y)   { return ((x)<(y))?0.0:1.0; }

static inline double _fn_pow(double x,double y)   { return pow(x,y); }
static inline double _d0_pow(double x,double y)   { return (x==0.0)?0.0:((y/x)*pow(x,y)); }
static inline double _d1_pow(double x,double y)   { return (x==0.0)?0.0:((log(x)/exp(0.0))*pow(x,y)); }

static inline double _fn_limexp(double arg)       { return ((arg)<(80))?(exp(arg)):(exp(80.0)*(1.0+(arg-80))); }
static inline double _d0_limexp(double arg)       { return ((arg)<(80))?(exp(arg)):(exp(80.0)); }

static inline double _fn_vt(double arg)           { return (1.3806503e-23 / 1.602176462e-19) * arg; }
static inline double _d0_vt(double arg)           { (void) arg; return (1.3806503e-23 / 1.602176462e-19); }

